import adsk.core, os

class Brick(object): 

    def __init__(self, fileName, length): 
        _resource_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')
        self.fileName = os.path.join(_resource_dir, fileName)
        self.length = length
        self.refBrick = self.importBrick(fileName)
        self.measurements = self.measureModel(self.refBrick)
        self.x = self.measurements['x']
        self.y = self.measurements['y']
        self.z = self.measurements['z']
        self.counter = 0
    
    def importBrick(self, fileName):
        app = adsk.core.Application.get()
        design = app.activeProduct
        rootComp = design.rootComponent

        importManager = app.importManager

        _resource_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'resources')
        brickFileName = os.path.join(_resource_dir, fileName)
        brickImportConfig = importManager.createIGESImportOptions(brickFileName)
        brickImportConfig.isViewFit = False
        return importManager.importToTarget2(brickImportConfig, rootComp).item(0)

    def measureModel(self, brick):
        """
        Places a brick to measure its outer dimensions and returns those as a dictionary.

        @rtype: dict
        @return: A dictionary containing the x,y,z dimensions of the used brick 
        """

        brickBody = brick.bRepBodies.item(0)
        d = dict()
        d['x'] = brickBody.boundingBox.maxPoint.x - brickBody.boundingBox.minPoint.x
        d['y'] = brickBody.boundingBox.maxPoint.y - brickBody.boundingBox.minPoint.y
        d['z'] = brickBody.boundingBox.maxPoint.z - brickBody.boundingBox.minPoint.z
        return d