#Author-Group 1 - MCP
#Description-This is addin that transformy an existing 3D model in Fusion to a model consisting of interlocking plastic bricks with a few clicks.

import adsk.core, adsk.fusion, traceback, os, gettext,  math, random
from .Brick import Brick

DEV_MODE = False

commandIdOnPanel = 'CommandOnPanel'
panelId = 'SolidCreatePanel'
bomPaletteID = 'bomPalette' 

# Unique IDs for UI elements 
profileSelectionInputId = 'profileSelection_'
brickSelectionInputId = 'brickSelection_'


# Universal brick dimensions 
brickLengthY = 1.6
brickLengthZ = 1.12
nippleLengthZ = 0.16
brickGridDistance = 0.8
layerOffsetX = brickGridDistance * 2 # Define how far every second layer is offset to its neighbors. 

appearances = []
# global set of event handlers to keep them referenced for the duration of the command
handlers = []
    
def commandDefinitionById(id):
    app = adsk.core.Application.get()
    ui = app.userInterface
    if not id:
        ui.messageBox(_('commandDefinition id is not specified'))
        return None
    commandDefinitions_ = ui.commandDefinitions
    commandDefinition_ = commandDefinitions_.itemById(id)
    return commandDefinition_

def commandControlByIdForPanel(id):
    app = adsk.core.Application.get()
    ui = app.userInterface
    if not id:
        ui.messageBox(_('commandControl id is not specified'))
        return None
    workspaces_ = ui.workspaces
    modelingWorkspace_ = workspaces_.itemById('FusionSolidEnvironment')
    toolbarPanels_ = modelingWorkspace_.toolbarPanels
    toolbarPanel_ = toolbarPanels_.itemById(panelId)
    toolbarControls_ = toolbarPanel_.controls
    toolbarControl_ = toolbarControls_.itemById(id)
    return toolbarControl_

def destroyObject(uiObj, tobeDeleteObj):
    if uiObj and tobeDeleteObj:
        if tobeDeleteObj.isValid:
            tobeDeleteObj.deleteMe()
        else:
            uiObj.messageBox(_('tobeDeleteObj is not a valid object'))

def run(context):
    ui = None
    try:
        app = adsk.core.Application.get()
        ui  = app.userInterface
        commandName = 'BrickMe'
        commandDescription = 'Create a brick model from a body.'
        commandResources = './resources'
        iconResources = './resources'

        # React to changes to Input fields
        class InputChangedHandler(adsk.core.InputChangedEventHandler):
            def __init__(self):
                super().__init__()
            def notify(self, args):
                try:
                    command = args.firingEvent.sender
                    cmdInput = args.input     
                    inputs = command.commandInputs
                    if command.parentCommandDefinition.id == commandIdOnPanel :
                        brickSelectionInput = inputs.itemById(brickSelectionInputId)
                        hasSelection = False
                        for elementID in range(brickSelectionInput.listItems.count): 
                            if brickSelectionInput.listItems.item(elementID).isSelected:
                                hasSelection = True
                        if not hasSelection:
                            brickSelectionInput.listItems.item(0).isSelected = True
                            ui.messageBox('You need to select at least one type of brick!')
                except:
                    if ui:
                        ui.messageBox('Input changed event failed: {}'.format(traceback.format_exc()))


        class CommandExecuteHandler(adsk.core.CommandEventHandler):
            def __init__(self):
                super().__init__()
            def notify(self, args):
                try:
                    command = args.firingEvent.sender
                    inputs = command.commandInputs
                    

                    # If the command is the confirmation of the settings panel, fill the selected element with bricks,  
                    if command.parentCommandDefinition.id == commandIdOnPanel :
                        availableBricks = []
                        element = inputs.itemById(profileSelectionInputId).selection(0).entity
                        # Read the user-selected brick types from the input and create a list of selected bricks.
                        brickSelectionInput = inputs.itemById(brickSelectionInputId)
                        for elementID in range(brickSelectionInput.listItems.count): 
                            if brickSelectionInput.listItems.item(elementID).isSelected:
                                availableBricks.insert(0, Brick("bricks/brick2x%d.iges" % (elementID+1), (elementID+1)))
                        handleBrickPlacementProcess(element, availableBricks)
                except:
                    if ui:
                        ui.messageBox('command executed failed: {}'.format(traceback.format_exc()))

        class CommandCreatedEventHandlerPanel(adsk.core.CommandCreatedEventHandler):
            def __init__(self):
                super().__init__() 
            def notify(self, args):
                try:
                    if DEV_MODE:
                        sketches = app.activeProduct.rootComponent.sketches
                        sketch = sketches.add(app.activeProduct.rootComponent.xYConstructionPlane)
                        setExampleElementsforDev(sketch) 
                    # Create palette for BOM
                    palette = ui.palettes.itemById(bomPaletteID)
                    if not palette:
                        palette = ui.palettes.add(bomPaletteID, 'Bill of Material', 'palette.html', True, True, True, 300, 200)
                        # Dock the palette to the right side of Fusion window.
                        palette.dockingState = adsk.core.PaletteDockingStates.PaletteDockStateRight
                    palette.isVisible = False


                    cmd = args.command
                    # Register Handlers                    
                    onExecute = CommandExecuteHandler()
                    cmd.execute.add(onExecute)
                    onInputChanged = InputChangedHandler()
                    cmd.inputChanged.add(onInputChanged)
                    cmd.isExecutedWhenPreEmpted = False
                    # Keep handlers referenced for future use
                    handlers.append(onExecute)
                    handlers.append(onInputChanged)

                    commandInputs_ = cmd.commandInputs
                    profileInput = commandInputs_.addSelectionInput(profileSelectionInputId, 'Body', 'Select a body!')
                    profileInput.addSelectionFilter('Bodies')

                    # Add a Dropdown menu for selction of available bricks.
                    dropdown = commandInputs_.addDropDownCommandInput(brickSelectionInputId, "Available Bricks", 2)
                    for i in range(9):
                        dropdown.listItems.add('Brick 2x%d' % (i+1), True)
                except:
                    if ui:
                        ui.messageBox('Panel command created failed: {}'.format(traceback.format_exc()))
        commandDefinitions_ = ui.commandDefinitions

        # add a command on create panel in modeling workspace
        workspaces_ = ui.workspaces
        modelingWorkspace_ = workspaces_.itemById('FusionSolidEnvironment')
        toolbarPanels_ = modelingWorkspace_.toolbarPanels
        toolbarPanel_ = toolbarPanels_.itemById(panelId) # add the new command under the CREATE panel
        toolbarControlsPanel_ = toolbarPanel_.controls
        toolbarControlPanel_ = toolbarControlsPanel_.itemById(commandIdOnPanel)
        if not toolbarControlPanel_:
            commandDefinitionPanel_ = commandDefinitions_.itemById(commandIdOnPanel)
            if not commandDefinitionPanel_:
                commandDefinitionPanel_ = commandDefinitions_.addButtonDefinition(commandIdOnPanel, commandName, commandDescription, commandResources)
            onCommandCreated = CommandCreatedEventHandlerPanel()
            commandDefinitionPanel_.commandCreated.add(onCommandCreated)
            # Keep handlers referenced for future use
            handlers.append(onCommandCreated)
            toolbarControlPanel_ = toolbarControlsPanel_.addCommand(commandDefinitionPanel_)
            toolbarControlPanel_.isVisible = True

    except:
        if ui:
            ui.messageBox('AddIn Start Failed: {}'.format(traceback.format_exc()))

# Unload the AddIn 
def stop(context):
    ui = None
    try:
        app = adsk.core.Application.get()
        ui = app.userInterface
        objArrayPanel = []

        # Collect Elements from Panel
        commandControlPanel_ = commandControlByIdForPanel(commandIdOnPanel)
        if commandControlPanel_:
            objArrayPanel.append(commandControlPanel_)

        # Collect Elements from the UI
        commandDefinitionPanel_ = commandDefinitionById(commandIdOnPanel)
        if commandDefinitionPanel_:
            objArrayPanel.append(commandDefinitionPanel_)

        # Destroy all collected Elements
        for obj in objArrayPanel:
            destroyObject(ui, obj)

        palette = ui.palettes.itemById(bomPaletteID)
        if palette:
            palette.deleteMe()
    except:
        if ui:
            ui.messageBox('AddIn Stop Failed: {}'.format(traceback.format_exc()))

def handleBrickPlacementProcess(element, availableBricks):
    """
    Takes a body or a profile to fill and a list of available bricks. Fille the element with the given bricks.
    @type  element: BRepBody or adsk.fusion.Profile
    @param element: The body or profile to be filled.
    @type  availableBricks: list of Brick
    @param availableBricks: List of bricks used to fill the given element.
    """
    #Deactivate Timeline to improve performance of brick placement
    app = adsk.core.Application.get()
    ui = app.userInterface
    adsk.fusion.Design.cast(app.activeProduct).designType = adsk.fusion.DesignTypes.DirectDesignType

    # Prepare a ProgressDialog that also allows the user to cancel the process.
    progressDialog = ui.createProgressDialog()
    progressDialog.cancelButtonText = 'Cancel'
    progressDialog.isBackgroundTranslucent = False
    progressDialog.isCancelButtonShown = True
    progressDialog.hide()

    allOccs = adsk.fusion.Design.cast(app.activeProduct).rootComponent.occurrences
    transform = adsk.core.Matrix3D.create()
    constructionComponent = allOccs.addNewComponent(transform)
    brickComponent = constructionComponent.component
    brickComponent.name = "BrickMe Model"

    # Check if the user-selected object is a body or a profile and handle it accordingly. 
    if element.objectType == adsk.fusion.BRepBody.classType():
        sliceBody(element, progressDialog, availableBricks, brickComponent)
    elif element.objectType == adsk.fusion.Profile.classType():
        fillSingleProfile(element, progressDialog, availableBricks, brickComponent)
    # After brick placement is completed, hide the ProcessDialog.
    progressDialog.hide()
    for brick in availableBricks:
        brick.refBrick.deleteMe()
    adsk.fusion.Design.cast(app.activeProduct).designType = adsk.fusion.DesignTypes.ParametricDesignType


def displayBOM(availableBricks): 
    """
    Displays a palette with a list of used bricks. 
    @type  availableBricks: list of Brick
    @param availableBricks: List of bricks used to fill the given element.
    """
    # Create the palette.
    app = adsk.core.Application.get()
    ui = app.userInterface
    palette = ui.palettes.itemById(bomPaletteID)
    palette.isVisible = True      
    # The palette receives a string, so brick count numbers need to be serialized.
    x = ""
    emptyCount = 0
    for i in (reversed(range(9))):
        if (i+1) == availableBricks[len(availableBricks)-1-i+emptyCount].length:
            x += "," + str(availableBricks[i-emptyCount].counter)
        else:
            # Fill in a 0 in the list for bricks that were not used.
            x += ",0"
            emptyCount += 1
    # Send data to the palette.
    palette.sendInfoToHTML('send', x[1:])

def sliceBody(body, progressDialog, availableBricks, parentComponent):
    """
    Takes a body, slices it to layers and filles each layer with bricks.

    @type  body: BRepBody
    @param body: The body to be sliced and filled.
    @type  progressDialog: ProgressDialog
    @param progressDialog: The progressDialog to be updated during execution.
    @type  availableBricks: list of Brick
    @param availableBricks: List of bricks used to fill the given element.
    @type  parentComponent: adsk.fusion.Component
    @param parentComponent: The component that should contain the bricks.
    """
    product = adsk.core.Application.get().activeProduct
    design = adsk.fusion.Design.cast(product)

    # Get the construction planes
    planes = parentComponent.constructionPlanes
    # Track current layer for the progress dialog.
    z = 0
    # Get the bounding box and its dimensions
    boundingBox = body.boundingBox
    initialZ = boundingBox.minPoint.z
    initialX = boundingBox.maxPoint.x - boundingBox.minPoint.x
    initialY = boundingBox.maxPoint.y - boundingBox.minPoint.y
    object_origin = adsk.core.Point3D.create(initialX, initialY, initialZ)
    # Calculate the number of necessary layers for the progress dialog.
    height = boundingBox.maxPoint.z - boundingBox.minPoint.z
    layers = int(height / (brickLengthZ-nippleLengthZ))
    progressDialog.show("Placing Bricks...", 'Created layer %v/%m',0,layers, 0)

    # Start the first layer at the bottom of the bounding box.
    offset = initialZ 
    # While the next layer would fit inside the bounding box, create layers
    while boundingBox.maxPoint.z > offset + 0.5*(brickLengthZ - nippleLengthZ):
        # Check for user issued cancelation. 
        if progressDialog.wasCancelled:
            break
        planeInput = planes.createInput()
        # Calculate the z position  of the next layer.
        offsetValue = adsk.core.ValueInput.createByReal(offset)
        # Create a new plane parallel to the x-y plane at the calculated z position. 
        planeInput.setByOffset(parentComponent.xYConstructionPlane, offsetValue)
        plane = planes.add(planeInput)
        # Create a sketch on the new plane
        sketches = parentComponent.sketches
        sketchme = sketches.add(plane)
        entities = []
        entities.append(body) 
        # Create a list of profiles by intersecting the body with the plane 
        sketchme.intersectWithSketchPlane(entities)
        # Fill all profiles of a single plane with bricks
        for profile in sketchme.profiles:
            # Shift every second layer for better stacking
            shift = False
            if z % 2 == 1:
                shift = True
            fillProfile(body, profile, offset, object_origin, availableBricks, parentComponent, z, shift)
        # Clean up helper objects
        sketchme.deleteMe()
        plane.deleteMe()

        # Update progress dialog after each layer.
        z += 1
        offset = initialZ + z*(brickLengthZ-nippleLengthZ)
        progressDialog.progressValue = z

    if not progressDialog.wasCancelled: 
        displayBOM(availableBricks)
    # Return the number of bricks used for this body.
    return True

def fillProfile(body, profile, zOffset, object_origin, availableBricks, parentComponent, layerID = 0, shift = False):
    """
    Takes a profile in the x-y plane and fills it with bricks.
    @type  body: BRepBody
    @param body: The body to be filled.
    @type  profile: adsk.fusion.Profile
    @param profile: The profile to be filled.
    @type  zOffset: number
    @param zOffset: The offset along the z axis where the plane should be created.
    @type  object_origin: adsk.core.Point3D
    @param object_origin: The point specifying the center of the whole body
    @type  parentComponent: adsk.fusion.Component
    @param parentComponent: The component that should contain the bricks.
    @type  layerID: number
    @param layerID: The name of the layer used as the component name.
    @type  shift: Boolean
    @param shift: Indicates if the brick representation should be shifte by half a brick-length across x and y to support stable stacking
    """

    # Get core information about the given profile 
    profileBox = profile.boundingBox
    areaProps = profile.areaProperties(adsk.fusion.CalculationAccuracy.MediumCalculationAccuracy)
    centroid = areaProps.centroid
    centroid.z = zOffset
    profileYSize = profileBox.maxPoint.y - profileBox.minPoint.y
    rowcount = int(profileYSize / brickLengthY)
    profileBricks = []

    x = 1
    y = 0
    
    # Compute the initial point on the brick grid to place the first brick, as well as its 2D projection. 
    centerPoint = getCenterPoint(y, zOffset, 0, 0, centroid, object_origin, shift)
    flatCenter = getPointWithZ0(centerPoint)

    # From the center, placement is done in positive Y direction an in negative Y direction seperately.
    for yDirection in range(2):
        # As long as the Bounding Box of the given profile contains the 2D projection of our next placement point, continue placing.
        while profileBox.contains(flatCenter):
            # As long we are inside the bounding box of the profile, fill each row (1D in x direction) seperately and sum up the returned brick counts.  
            profileBricks.extend(fillRow(y, yDirection, profileBox, centroid, body, zOffset, object_origin, availableBricks, shift))
            # Recompute center for next row. 
            y = y + 1
            centerPoint = getCenterPoint(y, zOffset, 0, yDirection, centroid, object_origin, shift)
            flatCenter = getPointWithZ0(centerPoint)
        # Reset after positive yDirection for negative yDirection.
        y = 1
        centerPoint = getCenterPoint(y, zOffset, 0, yDirection, centroid, object_origin, shift)
        flatCenter = getPointWithZ0(centerPoint)
    parentOccs = parentComponent.occurrences
    newChildComponent = parentOccs.addNewComponent(adsk.core.Matrix3D.create()).component
    newChildComponent.name = "Layer %d" % layerID
    bodies = newChildComponent.bRepBodies
    persistentBodies = []
    for brickTuple in profileBricks:
        brick, length = brickTuple
        body = bodies.add(brick)
        persistentBodies.append(body)
        colorBrick(body, length, availableBricks)
    return profileBricks

def fillRow(y, yDirection, boundigBox, centroid, body, zOffset, object_origin, availableBricks, shift_layer = False):
    """
    Determines the point where a brick should be placed, based on the center of the current profile and the global center of the object. 
    @type  y: number
    @param y: number of bricks between the center and the current brick in y direction
    @type  yDirection: number
    @param yDirection: 0, if we are currently moving from the center in positive y direction, 1 otherwise
    @type  boundingBox: adsk.BoundingBox3D
    @param boundingBox: The bounding box that determines the maximum dimensions of the row
    @type  centroid: adsk.core.Point3D
    @param centroid: The point specifying the center of the current profile
    @type  body: BRepBody
    @param body: The body to be filled. 
    @type  zOffset: number
    @param zOffset: The offset along the z axis where the plane should be created.
    @type  object_origin: adsk.core.Point3D
    @param object_origin: The point specifying the center of the whole body
    @type  shift_layer: Boolean
    @param shift_layer: Indicates if the current layer should be shifte by half a brick-length across x and y to support stable stacking
    """
    x = 0
    z = 0        
    rowBricks = []
    
    # From the center of th row, placement is done in positive X direction an in negative X direction seperately.
    for xDirection in range(2):
        # Get the initial point of placement, its 2D projection and a copy of the placement point used to determine if a certain brick fits within the profile.
        centerPoint = getCenterPoint(y, zOffset, 0, yDirection, centroid, object_origin, shift_layer)
        flatCenter = getPointWithZ0(centerPoint)
        newPoint = centerPoint.copy()

        # As long as the given bounding box contains the 2D projection of our next placement point, continue placing.
        while boundigBox.contains(flatCenter):
            # Check all available bricks and place the longest, that still fits within the profile.
            for i, brick in enumerate(availableBricks):
                # Track if a brick was placed this round
                brickPlaced = False
                brickFits = True
                # Since we move from the center in two directions, the next placement depends on the direction.
                if xDirection == 0:
                    # Check for each grid row of the brick if it covers a hole
                    testPoint = adsk.core.Point3D.create(newPoint.x + brickGridDistance / 2, newPoint.y + brickLengthY / 2, newPoint.z)
                    for j in range(brick.length):
                        if body.pointContainment(testPoint) >= 2:
                            brickFits = False
                            break
                        testPoint.x += brickGridDistance
                    newPoint.x += brick.x
                else:
                    # Check for each grid row of the brick if it covers a hole
                    testPoint = adsk.core.Point3D.create(newPoint.x - brickGridDistance / 2, newPoint.y + brickLengthY / 2, newPoint.z)
                    for j in range(brick.length):
                        if body.pointContainment(testPoint) >= 2:
                            brickFits = False
                            break
                        testPoint.x -= brickGridDistance
                    newPoint.x -= brick.x 
                    centerPoint = newPoint.copy()
                # If the end of the brick fits inside the body, place it.
                if brickFits:
                    newBrick, length = setBrickToLocation(brick, centerPoint.asVector())
                    rowBricks.append((newBrick, length))
                    # Set next placement point to the end of the current brick.
                    centerPoint = newPoint.copy()
                    flatCenter = getPointWithZ0(centerPoint)
                    brickPlaced = True
                    break
                # If the brick was not placed, reset the variable indicating the end of the current brick to the beginning of the brick.
                else:
                    if xDirection == 0:
                        newPoint.x -= brick.x
                    else:
                        newPoint.x += brick.x
                        centerPoint = newPoint.copy()
            # If no brick fits, move one step forward on the grid.
            if not brickPlaced: 
                if xDirection == 0:
                    newPoint.x += brickGridDistance
                    centerPoint.x += brickGridDistance
                    flatCenter = getPointWithZ0(centerPoint)
                else:
                    newPoint.x -= brickGridDistance
                    centerPoint.x -= brickGridDistance
                    flatCenter = getPointWithZ0(centerPoint)
    return rowBricks 

def getXPosition(y, xDirection, centroid, object_origin_x, shift_layer = False):
    """
    Determines the position in x direction where a brick should be placed, based on the brick grid and the center of the profile. 
    @type  y: number
    @param y: number of bricks between the center and the current brick in y direction
    @type  xDirection: number
    @param xDirection: 0, if we are currently moving from the center in positive x direction, 1 otherwise
    @type  centroid: adsk.core.Point3D
    @param centroid: The point specifying the center of the current profile
    @type  object_origin_x: number
    @param object_origin: The x coordinate specifying the center of the whole body
    @type  shift_layer: Boolean
    @param shift_layer: Indicates if the current layer should be shifte by half a brick-length to support stable stacking
    @rtype: number
    @return: The x coordinate that aligns with the brick grid for the placement of a brick, offset by x bricks.
    """
    center = round((centroid.asVector().x-object_origin_x)/brickGridDistance) * brickGridDistance + object_origin_x # guarantees correct stacking, by moving the profile center to the brick grid

    offsetX = center 
   # if y % 2 == 1 : # Optional: Offset individual rows of the same layer
   #     offsetX = offsetX + brickLengthY
    if shift_layer:
        offsetX -= layerOffsetX/2 
    return offsetX

def getYPosition(y, yDirection, centroid, object_origin_y, shift_layer = False):
    """
    Determines the position in y direction where a brick should be placed, based on the brick grid and the center of the profile. 
    @type  y: number
    @param y: number of bricks between the center and the current brick in y direction
    @type  yDirection: number
    @param yDirection: 0, if we are currently moving from the center in positive y direction, 1 otherwise
    @type  centroid: adsk.core.Point3D
    @param centroid: The point specifying the center of the current profile
    @type  object_origin_y: number
    @param object_origin: The y coordinate specifying the center of the whole body
    @type  shift_layer: Boolean
    @param shift_layer: Indicates if the current layer should be shifte by half a brick length to support stable stacking
    @rtype: number
    @return: The y coordinate that aligns with the brick grid for the placement of a brick, offset by y bricks.
    """
    center = round((centroid.asVector().y-object_origin_y)/brickGridDistance) * brickGridDistance + object_origin_y # guarantees correct stacking, by moving the profile center to the next brick grid position
    offsetY = center + y*(brickLengthY)
    if yDirection == 1:
        offsetY = center - y*(brickLengthY)
    if shift_layer: 
        offsetY += brickLengthY/2
    return offsetY

def getCenterPoint(y, z, xDirection, yDirection, centroid, object_origin, shift_layer=False):
    """
    Determines the point where a brick should be placed, based on the center of the current profile and the global center of the object. 

    @type  y: number
    @param y: number of bricks between the center and the current brick in y direction
    @type  z: number
    @param z: z-Position of the current layer
    @type  xDirection: number
    @param xDirection: 0, if we are currently moving from the center in positive x direction, 1 otherwise
    @type  yDirection: number
    @param yDirection: 0, if we are currently moving from the center in positive y direction, 1 otherwise
    @type  centroid: adsk.core.Point3D
    @param centroid: The point specifying the center of the current profile
    @type  object_origin: adsk.core.Point3D
    @param object_origin: The point specifying the center of the whole body
    @type  shift_layer: Boolean
    @param shift_layer: Indicates if the current layer should be shifte by half a brick-length across x and y to support stable stacking
    @rtype: adsk.core.Point3D
    @return: A point that aligns with the brick grid for the placement of a brick.
    """
    offsetX = getXPosition(y, xDirection, centroid, object_origin.x, shift_layer)
    offsetY = getYPosition(y, yDirection, centroid, object_origin.y, shift_layer)
    return adsk.core.Point3D.create(offsetX, offsetY, z)

def getPointWithZ0(point):
    """
    Takes a Point3D and returns its projection onto the x-y plane

    @type  point: adsk.core.Point3D
    @param point: Any point to be projected onto the x-y plane
    @rtype: adsk.core.Point3D
    @return: A copy of the provided point where the z coordinate is set to 0.
    """

    newPoint = point.copy()
    newPoint.z = 0
    return newPoint 

def setBrickToLocation(brick, location):
    """
    Imports a brick and moves it to a given location.

    @type  location: adsk.core.Vector3D
    @param location: The desired, translatory position of the created brick.
    @rtype:   adsk.fusion.Occurrence
    @return:  The occurence containing the placed brick.
    """
    app = adsk.core.Application.get()
    ui = app.userInterface
    product = app.activeProduct
    design = adsk.fusion.Design.cast(product)
    tempBrepMgr = adsk.fusion.TemporaryBRepManager.get()
    transform = adsk.core.Matrix3D.create()
    transform.translation = location
    brickLocal = brick.refBrick.component.bRepBodies.item(0)

    newBrick = tempBrepMgr.copy(brickLocal)
    tempBrepMgr.transform(newBrick, transform)
    adsk.doEvents()
    brick.counter += 1

    return  newBrick, brick.length

def colorBrick(brick, length, availableBricks):
    """
    Takes a brick and its length (in multiples of the brick grid length) and colors the brick based on its length.
    @type  brick: BRepBody or adsk.fusion.Profile
    @param brick: The brick to be colored.
    @type length:  Number
    @param length: The length of the brick in 2xX.
    @type  availableBricks: list of Brick
    @param availableBricks: List of bricks used to fill the given element.
    """
    try:
        brick.appearance = appearances[length-1]
    except:
        setColors(availableBricks[0].refBrick.component.bRepBodies.item(0))
        brick.appearance = appearances[length-1]

def setColors(templateBody):
    """
    Takes a BRepBody as a template and creates a list of appearances.
    @type  templateBody: adsk.fusion.BRepBody
    @param templateBody: A BRepBody. Its apperance is used as the template for all the appearances. 
    """
    global appearances
    appearances = []
    app = adsk.core.Application.get()
    design = app.activeProduct
    refBrickBody = templateBody
    refAppear = refBrickBody.appearance

    colors = []
    colors.append(adsk.core.Color.create(255, 0, 255, 0))
    colors.append(adsk.core.Color.create(64, 64, 64, 0))
    colors.append(adsk.core.Color.create(130, 130, 130, 0))
    colors.append(adsk.core.Color.create(255, 0, 0, 0))
    colors.append(adsk.core.Color.create(255, 255, 255, 0))
    colors.append(adsk.core.Color.create(0, 100, 0, 0))
    colors.append(adsk.core.Color.create(255, 250, 0, 0))
    colors.append(adsk.core.Color.create(0, 0, 255, 0))
    colors.append(adsk.core.Color.create(255, 0, 0, 0))

    for i in range(9):
        newColor = design.appearances.addByCopy(refAppear, "Brick%d Appearance" % (i+1))
        colorProp = adsk.core.ColorProperty.cast(newColor.appearanceProperties.itemByName('Color'))
        colorProp.value = colors[i]
        appearances.append(newColor)

def setExampleElementsforDev(sketch):
    body = createTorusFragment(sketch, math.pi, 10)

def createTorusFragment(sketch, angle, offset):
    app = adsk.core.Application.get()
    ui = app.userInterface

    circles = sketch.sketchCurves.sketchCircles
    circle1 = circles.addByCenterRadius(adsk.core.Point3D.create(offset, 0, 0), 3)
    
    lines = sketch.sketchCurves.sketchLines
    axis = lines.addByTwoPoints(adsk.core.Point3D.create(offset-1,-8,0), adsk.core.Point3D.create(offset +1,-8,0))

    prof = sketch.profiles.item(len(sketch.profiles)-1)

    revolves = app.activeProduct.rootComponent.features.revolveFeatures
    revInput = revolves.createInput(prof, axis, adsk.fusion.FeatureOperations.NewBodyFeatureOperation)
    angle = adsk.core.ValueInput.createByReal(angle)
    revInput.setAngleExtent(False, angle)
    rev = revolves.add(revInput)
    return rev.bodies.item(0)